package envoy.client.ui.settings;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;

import envoy.client.Settings;
import envoy.client.event.EventBus;
import envoy.client.event.ThemeChangeEvent;
import envoy.client.ui.Theme;
import envoy.client.util.EnvoyLog;

/**
 * This class provides the GUI to change the user specific settings.<br>
 * <br>
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>SettingsScreen.java</strong><br>
 * Created: <strong>31 Oct 2019</strong><br>
 *
 * @author Leon Hofmeister
 * @author Maximilian K&auml;fer
 * @author Kai S. K. Engelbart
 */
public class SettingsScreen extends JDialog {

	private static final long serialVersionUID = -4476913491263077107L;

	private final JPanel contentPanel = new JPanel();

	// Settings panel list
	private final DefaultListModel<String>	optionsListModel	= new DefaultListModel<>();
	private final JList<String>				options				= new JList<>(optionsListModel);

	// OK and cancel buttons
	private final JPanel	buttonPane		= new JPanel();
	private final JButton	okButton		= new JButton("Save");
	private final JButton	cancelButton	= new JButton("Cancel");

	private final int space = 5;

	private SettingsPanel settingsPanel;

	private static final Logger logger = EnvoyLog.getLogger(SettingsScreen.class.getSimpleName());

	/**
	 * Initializes the settings screen.
	 *
	 * @since Envoy v0.1-alpha
	 */
	public SettingsScreen() {
		logger.info("Currently selected theme: " + Settings.getInstance().getCurrentTheme());

		Map<String, Class<? extends SettingsPanel>> panels = new HashMap<>();
		panels.put("Color Themes", ThemeCustomizationPanel.class);

		setBounds(10, 10, 450, 650);
		getContentPane().setLayout(new BorderLayout());
		{
			// Content pane
			GridBagLayout gbl_contentPanel = new GridBagLayout();

			gbl_contentPanel.columnWidths	= new int[] { 1, 1 };
			gbl_contentPanel.rowHeights		= new int[] { 1 };
			gbl_contentPanel.columnWeights	= new double[] { 0.05, 1.0 };
			gbl_contentPanel.rowWeights		= new double[] { 1.0 };

			getContentPane().add(contentPanel, BorderLayout.CENTER);
			contentPanel.setLayout(gbl_contentPanel);

			options.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			options.addListSelectionListener((listSelectionEvent) -> {
				if (!listSelectionEvent.getValueIsAdjusting()) {
					// Get selected settings panel
					final String option = options.getSelectedValue();
					logger.log(Level.FINEST, "Selected settings panel: " + option);

					// Remove previous settings panel
					if (settingsPanel != null)
						contentPanel.remove(settingsPanel);

					try {
						settingsPanel = panels.get(option).getDeclaredConstructor().newInstance();

						// Add selected settings panel
						contentPanel.add(settingsPanel);
						revalidate();
						repaint();
					} catch (InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException
							| NoSuchMethodException | SecurityException e) {
						e.printStackTrace();
					}
				}
			});
			options.setFont(new Font("Arial", Font.PLAIN, 14));

			GridBagConstraints gbc_optionsList = new GridBagConstraints();
			gbc_optionsList.fill	= GridBagConstraints.BOTH;
			gbc_optionsList.gridx	= 0;
			gbc_optionsList.gridy	= 0;
			gbc_optionsList.anchor	= GridBagConstraints.PAGE_START;
			gbc_optionsList.insets	= new Insets(space, space, space, space);

			panels.keySet().forEach(name -> optionsListModel.addElement(name));
			contentPanel.add(options, gbc_optionsList);

			// ButtonPane
			GridBagLayout gbl_buttonPane = new GridBagLayout();
			gbl_buttonPane.columnWidths		= new int[] { 100, 250, 100, 0 };
			gbl_buttonPane.rowHeights		= new int[] { 25, 0 };
			gbl_buttonPane.columnWeights	= new double[] { 0.0, 0.0, 0.0, Double.MIN_VALUE };
			gbl_buttonPane.rowWeights		= new double[] { 0.0, Double.MIN_VALUE };

			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			buttonPane.setLayout(gbl_buttonPane);
			{
				cancelButton.setActionCommand("Cancel");
				cancelButton.setBorderPainted(false);
				GridBagConstraints gbc_cancelButton = new GridBagConstraints();
				gbc_cancelButton.anchor	= GridBagConstraints.NORTHWEST;
				gbc_cancelButton.insets	= new Insets(space, space, space, space);
				gbc_cancelButton.gridx	= 0;
				gbc_cancelButton.gridy	= 0;
				buttonPane.add(cancelButton, gbc_cancelButton);

				cancelButton.addActionListener((evt) -> { dispose(); });
			}
			{
				okButton.setActionCommand("OK");
				okButton.setBorderPainted(false);
				GridBagConstraints gbc_okButton = new GridBagConstraints();
				gbc_okButton.anchor	= GridBagConstraints.NORTHEAST;
				gbc_okButton.fill	= GridBagConstraints.EAST;
				gbc_okButton.insets	= new Insets(space, space, space, space);
				gbc_okButton.gridx	= 2;
				gbc_okButton.gridy	= 0;
				buttonPane.add(okButton, gbc_okButton);
				getRootPane().setDefaultButton(okButton);

				// Invoke settings panel action on button press
				okButton.addActionListener((evt) -> { if (settingsPanel != null) settingsPanel.getOkButtonAction().actionPerformed(evt); });
			}
		}

		// Apply current theme
		applyTheme(Settings.getInstance().getThemes().get(Settings.getInstance().getCurrentTheme()));

		// Respond to theme changes
		EventBus.getInstance().register(ThemeChangeEvent.class, (evt) -> applyTheme(((ThemeChangeEvent) evt).get()));

		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		setModal(true);
	}

	private void applyTheme(Theme theme) {
		// whole JDialog
		setBackground(theme.getBackgroundColor());
		// contentPanel
		contentPanel.setBackground(theme.getBackgroundColor());
		// buttonPane
		buttonPane.setBackground(theme.getCellColor());
		// cancelButton
		cancelButton.setBackground(theme.getInteractableBackgroundColor());
		cancelButton.setForeground(theme.getInteractableForegroundColor());
		// okButton
		okButton.setBackground(theme.getInteractableBackgroundColor());
		okButton.setForeground(theme.getInteractableForegroundColor());
		// options
		options.setSelectionForeground(theme.getUserNameColor());
		options.setSelectionBackground(theme.getSelectionColor());
		options.setForeground(theme.getUserNameColor());
		options.setBackground(theme.getCellColor());
	}
}
